@@ -3,9 +3,9 @@ import os
 import argparse
 import shutil
 from dataclasses import dataclass, asdict
-from typing import Optional
+from typing import List, Optional
 
-from .common import is_legal_local_lib_dir
+from .common import available_arches, available_build_names, available_platforms
 from . import common
 from dacite import from_dict
 from tabulate import tabulate
@@ -89,6 +89,31 @@ class LibRepo:
         if self.exist(lib):
             raise Exception('already exist')
         self.libs.append(lib)
+        self.push_repo()
+
+    def remove_lib(self, lib: LibData):
+        if not self.exist(lib):
+            raise Exception('Not exist')
+
+        self.libs.remove(lib)
+        self.push_repo()
+        common.remove_file(lib.file_key())
+
+    def bulk_remove_libs(self, libs: List[LibData]):
+        if not libs:
+            return
+
+        for l in libs:
+            if not self.exist(l):
+                raise Exception(f'Not exist, lib: {l}')
+
+        for l in libs:
+            self.libs.remove(l)
+        self.push_repo()
+
+        file_keys = [l.file_key() for l in libs]
+        common.bulk_remove_files(file_keys)
+
 
     def exist(self, lib: LibData):
         if not self.libs:
@@ -96,6 +121,27 @@ class LibRepo:
 
         return any(lb == lib for lb in self.libs)
 
+    def find_compatible_libs(self, name: str, ver: str, platform: str, arch=None, build=None):
+        libs = []
+        if not arch and not build:
+            for b in available_build_names():
+                for p in available_arches(platform):
+                    lib = LibData(name=name, version=ver, platform=platform, arch=p, build=b)
+                    if self.exist(lib):
+                        libs.append(lib)
+        if arch and not build:
+            for b in available_build_names():
+                lib = LibData(name=name, version=ver, platform=platform, arch=arch, build=b)
+                if self.exist(lib):
+                    libs.append(lib)
+        if not arch and build:
+            for p in available_arches(platform):
+                lib = LibData(name=name, version=ver, platform=platform, arch=p, build=build)
+                if self.exist(lib):
+                    libs.append(lib)
+
+        return libs
+
     def __str__(self):
         def sort_key(lib: LibData):
             return f'{lib.name}-{lib.version}-{lib.platform}-{lib.arch}-{lib.build}'
@@ -127,7 +173,6 @@ def update_lib(local_path: str, lib: LibData):
     repo.pull_repo()
     if not repo.exist(lib):
         repo.add_new_lib(lib)
-        repo.push_repo()
 
 
 def parse_args():
@@ -135,25 +180,31 @@ def parse_args():
     parser = argparse.ArgumentParser(description=description)
     sub_parsers = parser.add_subparsers(dest='sub_command')
     add_parser = sub_parsers.add_parser('add')
-    add_parser.add_argument('--path', help='The library folder you want to upload.')
-    add_parser.add_argument('--name', help='The library name.')
-    add_parser.add_argument('--version', help='The library version.')
-    add_parser.add_argument('--platform', help='Platform the library running.', choices=common.available_platforms())
-    add_parser.add_argument('--arch', help='The library architecture.', choices=common.all_available_arches())
-    add_parser.add_argument('--build', help='The library build type.', choices=common.available_build_names())
+    add_parser.add_argument('-p', '--path', help='The library folder you want to upload.', required=True)
+    add_parser.add_argument('-n', '--name', help='The library name.', required=True)
+    add_parser.add_argument('-v', '--version', help='The library version.')
+    add_parser.add_argument('-s', '--platform', help='Platform the library running.', choices=common.available_platforms())
+    add_parser.add_argument('-a', '--arch', help='The library architecture.', choices=common.all_available_arches())
+    add_parser.add_argument('-b', '--build', help='The library build type.', choices=common.available_build_names())
 
     remove_parser = sub_parsers.add_parser('remove')
-    remove_parser.add_argument('--name')
+    remove_parser.add_argument('-n', '--name', help='The library name.', required=True)
+    remove_parser.add_argument('-v', '--version', help='The library version.')
+    remove_parser.add_argument('-s', '--platform', help='Platform the library running.', choices=common.available_platforms())
+    remove_parser.add_argument('-a', '--arch', help='The library architecture.', choices=common.all_available_arches())
+    remove_parser.add_argument('-b', '--build', help='The library build type.', choices=common.available_build_names())
 
     return parser.parse_args()
 
 
-def lib_repo():
-    args = parse_args()
+def __strip_or_none(s):
+    if not s:
+        return None
+    s = s.strip()
+    return s if s else None
 
-    if args.sub_command != 'add':
-        exit(1)
 
+def execute_add(args):
     if not args.path:
         while True:
             path = input("Input the lib local path you want to upload: ")
@@ -176,13 +227,10 @@ def lib_repo():
         build = input("Input the build type for the lib you want to upload: ")
         args.build = build
 
-    def strip(s):
-        return s.strip() if s else None
-
-    lib_data = LibData(name=args.name, version=strip(args.version),
-                       platform=strip(args.platform),
-                       arch=strip(args.arch),
-                       build=strip(args.build))
+    lib_data = LibData(name=args.name, version=__strip_or_none(args.version),
+                       platform=__strip_or_none(args.platform),
+                       arch=__strip_or_none(args.arch),
+                       build=__strip_or_none(args.build))
 
     lib_repo = LibRepo().pull_repo()
     upload = True
@@ -202,6 +250,32 @@ def lib_repo():
         update_lib(args.path, lib_data)
 
 
+def execute_remove(args):
+    lib_repo = LibRepo().pull_repo()
+    platform = __strip_or_none(args.platform)
+    platforms = [platform] if platform else available_platforms()
+
+    all_libs = []
+    for p in platforms:
+        libs = lib_repo.find_compatible_libs(name=__strip_or_none(args.name), ver=__strip_or_none(args.version), platform=p, 
+                                            arch=__strip_or_none(args.arch), build=__strip_or_none(args.build))
+        if libs:
+            all_libs.extend(libs)
+
+    lib_repo.bulk_remove_libs(all_libs)
+    
+
+
+def lib_repo():
+    args = parse_args()
+
+    if args.sub_command == 'add':
+        execute_add(args)
+    elif args.sub_command == 'remove':
+        execute_remove(args)
+
+    
+
 if __name__ == "__main__":
     lib_repo()
 
